from AccessControl import ClassSecurityInfo
import Globals
from Products.CMFCore.utils import UniqueObject, SimpleItemWithProperties, getToolByName
from Products.MailmanAdaptor.config import ENABLED, UNKNOWN, BYUSER, BYADMIN, BYBOUNCE

class MailmanException(Exception): pass

class MailmanAdaptor(UniqueObject, SimpleItemWithProperties):
    """  Tool that aids the mailman integration
    """
    id        = 'mailman_adaptor'
    meta_type = 'MailmanAdaptor Tool'
    
    manage_options = SimpleItemWithProperties.manage_options

    security = ClassSecurityInfo()

    def __init__(self):
        self.mailman_member_types = []

    def getMailmanTypes(self):
        """ Return the current member types used for the MailmanAdaptor
        """
        return self.mailman_member_types

    def setMailmanTypes(self, mailman_member_types):
        """ Return the current member types used for the MailmanAdaptor
        """
        self.mailman_member_types = mailman_member_types

    def manage_makeChanges(self, mailman_member_types):
        """ Manage interface to change the MailmanAdaptor settings
        """
        self.setMailmanTypes(mailman_member_types)

    #
    # The readable interface
    #
    def getMembers(self):
        """Get the LCE for all the members of the mailing list."""
        mct = getToolByName(self,"member_catalog")
        query={'portal_type':self.mailman_member_types}
        res = mct.searchResults(query)
        addresses = [address.getEmail.lower() for address in res if address.getEmail is not '']
        uniqueAddresses = []
        [uniqueAddresses.append(wrd) for wrd in addresses if not uniqueAddresses.count(wrd)]
        return uniqueAddresses
        
    def getRegularMemberKeys(self):
        """Get the LCE for all regular delivery members (i.e. non-digest)."""
        return self.getMembers()

    def getDigestMemberKeys(self):
        """Get the LCE for all digest delivery members."""
        raise NotImplementedError

    def isMember(self, member):
        """Return 1 if member KEY/LCE is a valid member, otherwise 0."""
        key = member.lower()
        mct = getToolByName(self,"member_catalog")
        query={'portal_type':self.mailman_member_types, 'lce':key}
        addresses = mct.searchResults(query)
        if addresses:
            return 1
        return 0

    def __assertIsMember(self, member):
        if not self.isMember(member):
            raise MailmanException, member

    def getMemberKey(self, member):
        """Return the KEY for the member KEY/LCE.

        If member does not refer to a valid member, raise NotAMemberError.
        """
        self.__assertIsMember(member)
        return member.lower()

    def getMemberCPAddress(self, member):
        """Return the CPE for the member KEY/LCE.

        If member does not refer to a valid member, raise NotAMemberError.
        """
        key = member.lower()
        mct = getToolByName(self,"member_catalog")
        query={'portal_type':self.mailman_member_types, 'lce':key}
        addresses = mct.searchResults(query)
        if addresses:
            return addresses[0].getEmail
        raise MailmanException, member

    def getMemberCPAddresses(self, members):
        """Return a sequence of CPEs for the given sequence of members.

        The returned sequence will be the same length as members.  If any of
        the KEY/LCEs in members does not refer to a valid member, that entry
        in the returned sequence will be None (i.e. NotAMemberError is never
        raised).
        """
        request_length = len(members)
        mct = getToolByName(self,"member_catalog")
        query={'portal_type':self.mailman_member_types, 'lce':members}
        res = mct.searchResults(query)
        addresses =  [address.getEmail for address in res]
        actual_length = len(addresses)
        if actual_length != request_length:
            for i in range(request_length - actual_length):
                addresses.append(None)
        return addresses
        

    def authenticateMember(self, member, response):
        """Authenticate the member KEY/LCE with the given response.

        If the response authenticates the member, return a secret that is
        known only to the authenticated member.  This need not be the member's
        password, but it will be used to craft a session cookie, so it should
        be persistent for the life of the session.

        If the authentication failed return False.  If member did not refer to
        a valid member, raise NotAMemberError.

        Normally, the response will be the password typed into a web form or
        given in an email command, but it needn't be.  It is up to the adaptor
        to compare the typed response to the user's authentication token.
        """
        raise NotImplementedError

    def getMemberPassword(self, member):
        """Return the member's password.

        If the member KEY/LCE is not a member of the list, raise
        NotAMemberError.
        """
        raise NotImplementedError

    def getMemberLanguage(self, member):
        """Return the preferred language for the member KEY/LCE.

        The language returned must be a key in mm_cfg.LC_DESCRIPTIONS and the
        mailing list must support that language.

        If member does not refer to a valid member, the list's default
        language is returned instead of raising a NotAMemberError error.
        """
        raise NotImplementedError

    def getMemberOption(self, member, flag):
        """Return the boolean state of the member option for member KEY/LCE.

        Option flags are defined in Defaults.py.

        If member does not refer to a valid member, raise NotAMemberError.
        """
        raise NotImplementedError

    def getMemberName(self, member):
        """Return the full name of the member KEY/LCE.

        None is returned if the member has no registered full name.  The
        returned value may be a Unicode string if there are non-ASCII
        characters in the name.  NotAMemberError is raised if member does not
        refer to a valid member.
        """
        key = member.lower()
        mct = getToolByName(self,"member_catalog")
        query={'portal_type':self.mailman_member_types, 'lce':key}
        addresses = mct.searchResults(query)
        if addresses:
            return addresses[0].getFullName
        else:
            raise MailmanException, member

    def getMemberTopics(self, member):
        """Return the list of topics this member is interested in.

        The return value is a list of strings which name the topics.
        """
        raise NotImplementedError

    def getDeliveryStatus(self, member):
        """Return the delivery status of this member.

        Value is one of the module constants:

            ENABLED  - The deliveries to the user are not disabled
            UNKNOWN  - Deliveries are disabled for unknown reasons.  The
                       primary reason for this to happen is that we've copied
                       their delivery status from a legacy version which didn't
                       keep track of disable reasons
            BYUSER   - The user explicitly disable deliveries
            BYADMIN  - The list administrator explicitly disabled deliveries
            BYBOUNCE - The system disabled deliveries due to bouncing

        If member is not a member of the list, raise NotAMemberError.
        """
        key = member.lower()
        mct = getToolByName(self,"member_catalog")
        query={'portal_type':self.mailman_member_types, 'lce':key}
        addresses = mct.searchResults(query)
        if addresses:
            return addresses[0].getDeliveryStatus
        else:
            raise MailmanException, member

    def getDeliveryStatusChangeTime(self, member):
        """Return the time of the last disabled delivery status change.

        If the current delivery status is ENABLED, the status change time will
        be zero.  If member is not a member of the list, raise
        NotAMemberError.
        """
        key = member.lower()
        mct = getToolByName(self,"member_catalog")
        query={'portal_type':self.mailman_member_types, 'lce':key}
        addresses = mct.searchResults(query)
        if addresses:
            return addresses[0].getDeliveryStatusChangeTime
        else:
            raise MailmanException, member

    def removeMember(self, memberkey):
        """Unsubscribes the member from the mailing list.

        Raise NotAMemberError if member is not subscribed to the list.
        """
        raise NotImplementedError

    def setMemberName(self, member, realname):
        """Set the member's full name.

        member is an LCE/KEY and realname is an arbitrary string.  It should
        be a Unicode string if there are non-ASCII characters in the name.
        NotAMemberError is raised if member does not refer to a valid member.
        """
        key = member.lower()
        mct = getToolByName(self,"member_catalog")
        query={'portal_type':self.mailman_member_types, 'lce':key}
        addresses = mct.searchResults(query)
        if addresses:
            obj = addresses[0].getObject()
            obj.setFullName(realname)
        else:
            raise MailmanException, member

    def setDeliveryStatus(self, member, status):
        """Set the delivery status of the member's address.

        Status must be one of the module constants:

            ENABLED  - The deliveries to the user are not disabled
            UNKNOWN  - Deliveries are disabled for unknown reasons.  The
                       primary reason for this to happen is that we've copied
                       their delivery status from a legacy version which didn't
                       keep track of disable reasons
            BYUSER   - The user explicitly disable deliveries
            BYADMIN  - The list administrator explicitly disabled deliveries
            BYBOUNCE - The system disabled deliveries due to bouncing

        This method also records the time (in seconds since epoch) at which
        the last status change was made.  If the delivery status is changed to
        ENABLED, then the change time information will be deleted.  This value
        is retrievable via getDeliveryStatusChangeTime().
        """
        key = member.lower()
        mct = getToolByName(self,"member_catalog")
        query={'portal_type':self.mailman_member_types, 'lce':key}
        addresses = mct.searchResults(query)
        if addresses:
            obj = addresses[0].getObject()
            obj.setDeliveryStatus(status)
        else:
            raise MailmanException, member

Globals.InitializeClass(MailmanAdaptor)
